# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.contrib import admin
from django.contrib.contenttypes import generic
from django_zotero.forms import TagInlineFormsetAdmin, TagInlineFormAdmin
from django_zotero.models import Field, ItemType, Tag, Document
from django_zotero.views import itemtype_fields_view


# schema
class FieldInlineAdmin(admin.TabularInline):
    model = ItemType.fields.through
    extra = 0


class FieldAdmin(admin.ModelAdmin):
    inlines = (
        FieldInlineAdmin,
    )
    list_display = (
        '__unicode__',
        'multiple',
    )


class ItemTypeAdmin(admin.ModelAdmin):
    inlines = (
        FieldInlineAdmin,
    )
    exclude = (
        'fields',
    )
    
    def get_urls(self):
        urls = super(ItemTypeAdmin, self).get_urls()
        field_urls = patterns('',
            url(
                r'^fields/$',
                self.admin_site.admin_view(
                    itemtype_fields_view,
                    cacheable=True,
            ),
            name='zotero_itemtype_fields'),
        )
        return field_urls + urls

admin.site.register(ItemType, ItemTypeAdmin)
admin.site.register(Field, FieldAdmin)
admin.site.register(Tag)


# data
class TagInlineAdmin(generic.GenericTabularInline):
    model = Tag
    extra = 0
    form = TagInlineFormAdmin
    formset = TagInlineFormsetAdmin
    template = 'admin/edit_inline/extended_tabular.html'


# test
class DocumentAdmin(admin.ModelAdmin):
    inlines = (
        TagInlineAdmin,
    )
admin.site.register(Document, DocumentAdmin)



# -*- coding: utf-8 -*-
from django import forms
from django.contrib.contenttypes import generic
from django.db import transaction
from models import Tag


class GenericTagInlineFormset(generic.BaseGenericInlineFormSet):
    def clean(self):
        single_fields = []
        first_loop = True
        for form in self.forms:
            try:
                item_type = form.cleaned_data['item_type']
                field = form.cleaned_data['field']
                delete = form.cleaned_data.get('DELETE')
            except AttributeError:
                pass
            except KeyError:
                pass
            else:
                if first_loop:
                    first_item_type = item_type
                    first_loop = False
                if not delete:
                    if first_item_type != item_type:
                        raise forms.ValidationError('The item type %s is\
                            different from the original item type %s.' %
                            (item_type, first_item_type))
                    if not field in item_type.fields.all():
                        raise forms.ValidationError('The field %s is not\
                            applicable to the item type %s.' %
                            (field, item_type))
                    if not field.multiple:
                        if field in single_fields:
                            raise forms.ValidationError('The field %s has\
                                multiple values.' % field)
                        else:
                            single_fields.append(field)
    
#    @transaction.set_autocommit
    def save(self):
        try:
            super(GenericTagInlineFormset, self).save()
        except TypeError, e:
            if e.message[0].startswith('Uniqueness'):
                tag = self.forms[0].instance
                new_item_type = tag.item_type
                obj = Tag.get_object(tag)
                tags = Tag.get_tags(obj)
                tags.update(item_type=new_item_type)
                self.save()


class TagInlineFormsetAdmin(GenericTagInlineFormset):
    pass


class TagInlineFormAdmin(generic.ModelForm):
    class Media:
        js = (
            'js/tags_admin.js',
        )


class TagInlineFormset(GenericTagInlineFormset):
    pass


class TagInlineFormNoJQuery(generic.ModelForm):
    class Media:
        js = (
#            'js/jquery.js',
            'js/jquery.formset.js',
            'js/tags.js',
        )
        css = {
            'all': ('css/tags.css',)
        }


class TagInlineForm(generic.ModelForm):
    class Media:
        js = (
            'js/jquery.js',
            'js/jquery.formset.js',
            'js/tags.js',
        )
        css = {
            'all': ('css/tags.css',)
        }


def get_tag_formset(obj=None, data=None, show_labels=False, labels=None, jquery=True):
    if obj and Tag.get_tags(obj):
        extra = 0
    else:
        extra = 1
    if jquery:
        tagInlineForm = TagInlineForm
    else:
        tagInlineForm = TagInlineFormNoJQuery
    Formset = generic.generic_inlineformset_factory(
        Tag,
        form=tagInlineForm,
        formset=TagInlineFormset,
        extra=extra,
    )
    formset = Formset(instance=obj, data=data)
    formset.show_labels = show_labels
    formset.item_type_label = 'Item type'
    formset.field_label = 'Field'
    formset.value_label = 'Value'
    if labels:
        formset.item_type_label = labels.get('item_type', 'Item type')
        formset.field_label = labels.get('field', 'Field')
        formset.value_label = labels.get('value', 'Value')
        for label in labels:
            field = formset.form.base_fields.get(label)
            if field:
                field.label = labels[label]
    return formset


#test
from models import Document
class DocumentForm(forms.ModelForm):
    class Meta:
        model = Document



# -*- coding: utf-8 -*-
from django import template as t
from django.template import loader


register = t.Library()


@register.tag
def zotero_inline_tags(parser, token):
    """
    Render an inline formset of tags.
    
    Usage:
        {% zotero_inline_tags formset[ option] %}
        option = "all" | "media" | "formset"
    """
    
    args = token.split_contents()
    length = len(args)
    if length == 2:
        rendered_node = RenderedAllNode(args[1])
    elif length == 3 and args[2].lower() == u'all':
        rendered_node = RenderedAllNode(args[1])
    elif length == 3 and args[2].lower() == u'media':
        rendered_node = RenderedMediaNode(args[1])
    elif length == 3 and args[2].lower() == u'formset':
        rendered_node = RenderedFormsetNode(args[1])
    else:
        raise t.TemplateSyntaxError('Incorrect arguments in %s.' % args[0])
    
    return rendered_node


class RenderedNode(t.Node):
    def __init__(self, formset):
        self.formset = t.Variable(formset)


class RenderedAllNode(RenderedNode):
    def render(self, context):
        formset = self.formset.resolve(context)
        template = loader.get_template('zotero/inline_tags.html')
        c = t.Context({'formset': formset, 'media': True})
        return template.render(c)


class RenderedMediaNode(RenderedNode):
    def render(self, context):
        formset = self.formset.resolve(context)
        return formset.media.render()


class RenderedFormsetNode(RenderedNode):
    def render(self, context):
        formset = self.formset.resolve(context)
        template = loader.get_template('zotero/inline_tags.html')
        c = t.Context({'formset': formset, 'media': False})
        return template.render(c)



# -*- coding: utf-8 -*-
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _
from jsonfield import JSONField

from utils import humanize


#schema
class Field(models.Model):
    """
    Field in a document: title, publisher, author, etc.
    """
    field_name = models.CharField(_(u'name'), max_length=100)
    multiple = models.BooleanField(_(u'allows multiple?'))
    namespaces = JSONField(_(u'namespaces'), blank=True)
    
    def __unicode__(self):
        return humanize(self.field_name)
    
    def get_item_types(self):
        return self.item_types.all()
    
    class Meta:
        ordering = ('field_name',)


class ItemType(models.Model):
    """
    Type of document: book, journal article, etc.
    """
    type_name = models.CharField(_(u'name'), max_length=100)
    fields = models.ManyToManyField(Field, related_name='item_types',
        verbose_name=_(u'fields'))
    
    def __unicode__(self):
        return humanize(self.type_name)
    
    def get_fields(self):
        return self.fields.all()


#data
class Tag(models.Model):
    """
    A description (item_type, field, value) for an object.
    """
    #Generic foreign key to item
    content_type = models.ForeignKey(ContentType,
        verbose_name=_(u'content type'))
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    #Tag's fields
    item_type = models.ForeignKey(ItemType, related_name='tags',
        verbose_name=_(u'item type'))
    field = models.ForeignKey(Field, related_name='tags',
        verbose_name=_(u'field'))
    value = models.CharField(_(u'value'), max_length=256)
    
    def __unicode__(self):
        return u'%s: %s=%s' % (self.item_type, self.field, self.value)
    
    @staticmethod
    def get_tags(obj):
        ct = ContentType.objects.get_for_model(obj)
        tags = Tag.objects.filter(content_type__pk=ct.id, object_id=obj.id)
        return tags
    
    def get_object(self):
        return self.content_object
    
    def set_object(self, obj):
        self.object_id = obj.id
        self.content_object = obj
        self.save()


#test
class Document(models.Model):
    name = models.CharField(_('name'), max_length=256)
    
    def __unicode__(self):
        return self.name



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

from django_zotero import views

urlpatterns = patterns('',
    url(r'^itemtypes/valid/$',
        views.valid_zotero_itemtypes_fields,
        name='valid_zotero_itemtypes_fields'),
    url(r'^itemtype/fields/$',
        views.itemtype_fields_view,
        name='zotero_itemtype_fields'),
    
    url(r'^$', views.index, name='index'),
    url(r'^(?P<doc_id>\d+)/$', views.detail, name='detail'),
)



# -*- coding: utf-8 -*-
import json
from django.conf import settings
from django.http import HttpResponse
from django_zotero.models import ItemType, Field
from os.path import abspath, dirname, join, pardir


def itemtype_fields_view(request):
    itemtype_id = request.GET.get("itemtype")
    obj_item_type = ItemType.objects.get(id=itemtype_id)
    fields = obj_item_type.get_fields()
    field_ids = [f.id for f in fields]
    json_fields = json.dumps(field_ids)
    return HttpResponse(json_fields, content_type='text/javascript')


def valid_zotero_itemtypes_fields(request):
    path = abspath(
        join(dirname(__file__),
        'media/valid_zotero_itemtypes_and_fields.txt'))
    itemtypes_fields = {}
    for line in open(path):
        if line and not line.startswith('#'):
            line = line.strip()
            itemtype = line.split(':')[0]
            fields = line.split(':')[1]
            fields = fields.split(',')
            itemtype_id = ItemType.objects.get(type_name=itemtype).id
            field_ids = [Field.objects.get(field_name=field).id for field in fields]
            itemtypes_fields[itemtype_id] = field_ids
    json_itemtypes_fields = json.dumps(itemtypes_fields)
    return HttpResponse(json_itemtypes_fields, content_type='text/javascript')


#test
from django.shortcuts import render
from django_zotero.forms import DocumentForm, get_tag_formset
from django_zotero.models import Document

def index(request):
    documents = Document.objects.all()
    context = {'documents': documents}
    return render(request, 'zotero/document/index.html', context)

def detail(request, doc_id):
    obj = Document.objects.get(pk=doc_id)
    
    form = DocumentForm(instance=obj)
    formset = get_tag_formset(obj, show_labels=False, labels={'item_type': 'Document type'})
    if request.POST:
        form = DocumentForm(instance=obj, data=request.POST)
        formset = get_tag_formset(obj, data=request.POST, labels={'item_type': 'Document type'})
        if form.is_valid() and formset.is_valid():
            form.save()
            formset.save()
    
    return render(request, 'zotero/document/detail.html', {'form2': form, 'formset': formset})



# -*- coding: utf-8 -*-
from django.template import Library, Node, TemplateSyntaxError, Variable
from django_zotero.models import Tag

register = Library()


@register.tag
def zotero_tags(parser, token):
    """
    Returns the code to be translated by Zotero.
    Usage:
        {% zotero_tags
            object=object
            vocabulary=vocabulary
            output_method=output_method %}
    Example:
        {% zotero_tags object=document1 vocabulary="dc" output_method="meta" %}
    """
    default = {
        'object': u'',
        'vocabulary': u'dc',
        'output_method': u'meta'
    }
    
    letters = ['zero', 'one', 'two', 'three', 'four', 'five', 'six', 'seven']
    min_args = 1
    max_args = len(default)
    if min_args == 1:
        min_pl = ''
    else:
        min_pl = 's'
    if max_args == 1:
        max_pl = ''
    else:
        max_pl = 's'
    args = token.split_contents()
    length = len(args)
    if length < min_args + 1:
        raise TemplateSyntaxError('%s requires at least %s argument%s.' %
                                  (args[0], letters[min_args], min_pl))
    elif length > max_args + 1:
        raise TemplateSyntaxError('%s requires %s or less argument%s.' %
                                  (args[0], letters[max_args], max_pl))
    
    for arg in args[1:length]:
        i = arg.find('=')
        if i == -1:
            raise TemplateSyntaxError('%s syntax error: %s.' % (args[0], arg))
        else:
            key = arg[:i]
            val = arg[i + 1:]
            if not key in default.keys():
                raise TemplateSyntaxError('%s invalid argument: %s.' %
                                          (args[0], key))
            else:
                default[key] = val
    
    return ZoteroTagsNode(default)


class ZoteroTagsNode(Node):
    def __init__(self, dictionary):
        self.obj = Variable(dictionary['object'])
        self.voc = Variable(dictionary['vocabulary'])
        self.out = Variable(dictionary['output_method'])
    
    def render(self, context):
        out = self.out.resolve(context)
        if out == 'meta':
            obj = self.obj.resolve(context)
            voc = self.voc.resolve(context)
            result = render_meta(obj, voc)
        else:
            pass
        return result


def render_meta(obj, vocabulary):
    result = u'<link rel="schema.dc" href="http://purl.org/dc/elements/1.1/">'
    
    tags = Tag.get_tags(obj)
    if tags:
        value = tags[0].item_type.type_name
        meta_tag = u'<meta property="DC.type" content="%s"/>' % value
        result = u'%s\n%s' % (result, meta_tag)
        for tag in tags:
            field = tag.field
            name = field.namespaces.get(vocabulary, field.field_name)
            value = tag.value
            if name == 'identifier':
                value = '%s %s' % (field.field_name.lower(), value)
            meta_tag = u'<meta property="DC.%s" content="%s"/>' % (name, value)
            result = u'%s\n%s' % (result, meta_tag)
    return result



# -*- coding: utf-8 -*-
from django.db.models.signals import pre_save, post_delete
from django.dispatch import receiver
from django_zotero.models import Tag


@receiver(pre_save, sender=Tag)
def check_save(sender, **kwargs):
    """
    Checks item type uniqueness, field applicability and multiplicity.
    """
    tag = kwargs['instance']
    obj = Tag.get_object(tag)
    previous_tags = Tag.get_tags(obj)
    
    err_uniq = check_item_type_uniqueness(tag, previous_tags)
    err_appl = check_field_applicability(tag)
    err_mult = check_field_multiplicity(tag, previous_tags)
    err_msg = generate_error_message(tag, err_uniq, err_appl, err_mult)
    if err_uniq or err_appl or err_mult:
        raise TypeError(err_msg)


def check_item_type_uniqueness(tag, previous_tags):
    """
    Check the uniqueness of the 'item type' for an object.
    """
    fail = False
    #If the tag is being created...
    if not tag.id:
        #... and the new item type is different from previous item types (for
        #example, different from the first of them), fail
        fail = previous_tags and tag.item_type != previous_tags[0].item_type
    #If the tag is being modifying...
    else:
        #... but there is only one previous tag (the one that is being
        #modifying), do not fail
        fail = previous_tags.count() > 1 and \
               tag.item_type != previous_tags[0].item_type
    return fail


def check_field_applicability(tag):
    """
    Check the applicability of a 'field' for an object.
    """
    #If the new field does not belong to the applicable field list, fail
    return not tag.field in tag.item_type.fields.all()


def check_field_multiplicity(tag, previous_tags):
    """
    Check the multiplicity of a 'field' for an object.
    """
    fail = False
    #If the field is single
    if not tag.field.multiple:
        #If the tag is being created...
        if not tag.id:
            #... and the new field was already included in the previous tags,
            #fail
            fail = previous_tags.filter(field=tag.field)
        #If the tag is being modifying...
        else:
            #... but there is only one previous tag (the one that is being
            #modifying), do not fail
            fail = previous_tags.filter(field=tag.field).count() > 1
    return fail


def generate_error_message(tag, err_uniq, err_appl, err_mult):
    """
    Generate the error message for an object.
    """
    err = []
    if err_uniq:
        err.append('Uniqueness restriction: item type %s' % tag.item_type)
    if err_appl:
        err.append('Applicability restriction: field %s' % tag.field)
    if err_mult:
        err.append('Multiplicity restriction: field %s' % tag.field)
    return err


@receiver(post_delete)
def delete_tags(sender, **kwargs):
    """
    Delete the tags pointing to an object.
    """
    try:
        obj = kwargs.get('instance')
        tags = Tag.get_tags(obj)
        tags.delete()
    except AttributeError:
        pass



import codecs
import re
from os import path
from setuptools import setup, find_packages


def read(*parts):
    file_path = path.join(path.dirname(__file__), *parts)
    return codecs.open(file_path).read()


def find_version(*parts):
    default = '0.3'
    try:
        version_file = read(*parts)
        version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                                  version_file, re.M)
        if version_match:
            return version_match.group(1)
        else:
            return default
    except:
        return default
#    raise RuntimeError("Unable to find version string.")


setup(
    name='django-zotero',
    version=find_version('django_zotero', '__init__.py'),
    author='Antonio Jimenez Mavillard',
    author_email='ajmavillard@gmail.com',
    url='https://github.com/CulturePlex/django-zotero',
    description='Django tool to tag objects and export them to Zotero',
    long_description=read('README.rst'),
    license='MIT',
    keywords='zotero django admin tag meta metatag',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: JavaScript',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        ],
    zip_safe=False,
    packages=['django_zotero'],
    include_package_data=True,
    install_requires=['jsonfield'],
)



def humanize(s):
    result = ''
    if s.islower() or s.isupper():
        result = s
    else:
        for c in s:
            if c.isupper():
                result = '%s %s' % (result.rstrip(), c.lower())
            else:
                result = '%s%s' % (result, c)
    result = result.strip()
    return result

def render_fields():
    from django_zotero.models import ItemType, Field
    result = ''
    for item_type in ItemType.objects.all():
        result += '\n*** ' + item_type.type_name + ' ***\n'
        result += '\n-------------------------------------------------------\n'
        result += u'<link rel="schema.dc" href="http://purl.org/dc/elements/1.1/">'
        value = item_type.type_name
        meta_tag = u'<meta property="DC.type" content="%s"/>' % value
        result = u'%s\n%s' % (result, meta_tag)
        for field in item_type.get_fields():
            name = field.namespaces.get('dc', field.field_name)
            value = field.field_name
            if name == 'identifier':
                value = '%s %s' % (field.field_name.lower(), value)
            meta_tag = u'<meta property="DC.%s" content="%s"/>' % (name, value)
            result = u'%s\n%s' % (result, meta_tag)
        result += '\n=======================================================\n'
    print result
    f = open('item_types_and_fields.txt', 'w')
    f.write(result)
    f.close
